@@ -4,17 +4,32 @@ use Moose::Role;
 
 requires qw/write_integer write_long/;
 
+use DateTime;
+use DateTime::Format::Strptime;
+use DateTime::Format::Epoch;
+
 sub write_date  {    #{{{
-    my ($self, $epoch_time) = @_;
-    my $time =
-      $epoch_time <= 4_294_967_295
-      ? $self->write_integer($epoch_time)
-      : $self->write_long($epoch_time);
+    my ($self, $datetime) = @_;
+    my $formatter = DateTime::Format::Epoch->new(
+        unit  => 'milliseconds',
+        type  => 'bigint',
+        epoch => DateTime->new(
+            year      => 1970,
+            month     => 1,
+            day       => 1,
+            time_zone => 'UTC'
+        )
+    );
+    my $epoch_time   = $formatter->format_datetime($datetime);
+    my $time = $self->write_long($epoch_time);
+#      $epoch_time <= 4_294_967_295
+#      ? $self->write_integer($epoch_time)
+#      : $self->write_long($epoch_time);
     if ($self->version() == 1) {
-        $time =~ s/^(?:I|L )/d/;
+        $time =~ s/L/d/;
     }
     else {
-        $time =~ s/^L/\x4a/;
+        $time =~ s/^L/d/;
         $time =~ s/^I/\x4b/;
 
     }
