@@ -24,15 +24,8 @@ sub t005_initialize_client : Test(1) {    #{{{
 
 sub compare_date {    #{{{
     my ( $self, $original_date, $processed_time ) = @_;
-    my $from_hessian_date = DateTime->from_epoch( epoch => $processed_time );
-    my $formatter = DateTime::Format::Strptime->new(
-        pattern   => '%F %T',
-        time_zone => 'GMT'
-    );
-    my $readable_date = $formatter->format_datetime($from_hessian_date);
-    $from_hessian_date->set_time_zone('UTC');
 
-    my $cmp = DateTime->compare( $original_date, $from_hessian_date );
+    my $cmp = DateTime->compare( $original_date, $processed_time );
     is( $cmp, 0, "Hessian date as expected." );
 }    #}}}
 
