@@ -9,6 +9,7 @@ use base 'Datatype::Message';
 use Test::More;
 use Test::Deep;
 use YAML;
+use Data::Dumper;
 use Hessian::Translator;
 use Hessian::Deserializer;
 use Hessian::Translator::Composite;
@@ -37,12 +38,12 @@ sub t004_initialize_hessian : Test(3) {    #{{{
 sub t010_read_hessian_version : Test(1) {    #{{{
     my $self         = shift;
     my $deserializer = $self->{deserializer};
-    my $hessian_data = "H\x02\x00";
+    my $hessian_data = "c\x02\x00";
     $deserializer->input_string($hessian_data);
     my $result = $deserializer->deserialize_message();
     cmp_deeply(
         $result,
-        { hessian_version => "2.0" },
+        superhashof( { hessian_version => "2.0" } ),
         "Parsed hessian version 2."
     );
 }    #}}}
@@ -50,41 +51,55 @@ sub t010_read_hessian_version : Test(1) {    #{{{
 sub t015_read_envelope : Test(2) {    #{{{
     my $self         = shift;
     my $deserializer = $self->{deserializer};
-    my $hessian_data = "H\x02\x00E\x06Header\x90\x87R\x05hello\x90Z";
+    my $hessian_data =
+      "E\x02\x00m\x00\x08Identity\x90" . "B\x00\x0ar\x02\x00\x05helloz\x90z";
     $deserializer->input_string($hessian_data);
     my $tokens = $deserializer->process_message();
+
     cmp_deeply(
-        $tokens->[0],
-        { hessian_version => "2.0" },
+        $tokens,
+        superhashof( { hessian_version => "2.0" } ),
         "Parsed hessian version 2."
     );
-    my $reply_data = $tokens->[1]->[0]->{packets}->[0];
-    my $text       = "$reply_data->{reply_data}";
-    cmp_deeply(
-        $reply_data,
-        superhashof( { reply_data => 'hello' } ),
-        "Parsed expected datastructure."
-    );
 
+    #    my $packet = $tokens->[0]->{envelope}->{packet};
+    my $packet = $tokens->{envelope}->{packet};
+    if ($packet) {
+        is( $packet, 'hello',
+            "Retrieved correct answer from enveloped reply." );
+    }
 }    #}}}
 
 sub t016_multi_chunk_envelope : Test(1) {    #{{{
     my $self         = shift;
     my $deserializer = $self->{deserializer};
-    my $hessian_data = "H\x02\x00E\x06Header\x90\x88C\x05hello"
-      . "\x91\x90\x90\x8d\x0chello, world\x90Z";
+    my $hessian_data =
+        "E\x02\x00m\x00\x08Identity" . "\x90"
+      . "B\x00\x0e"
+      . "p\x02\x00"
+      . "c\x02\x00m\x00\x04add2" . "z" . "\x90" . "\x90"
+      . "B\x00\x07"
+      . "p\x02\x00"
+      . "\x92\x93z" . "z" . "\x90" . "z";
     $deserializer->input_string($hessian_data);
-    my $tokens        = $deserializer->process_message();
-    my $second_packet = $tokens->[1]->[1]->{packets}->[0];
-    my $text          = "$second_packet";
-    cmp_deeply( $second_packet, "hello, world",
-        "Parsed expected datastructure." );
+    my $tokens = $deserializer->process_message();
+    my $packet = $tokens->{envelope}->{packet};
+    my $call;
+    if ($packet) {
+        $call = $packet->{call};
+    }
+
+    cmp_deeply(
+        $call,
+        { method => 'add2', arguments => [ 2, 3 ] },
+        "Parsed call from envelope."
+    );
 }    #}}}
 
-sub t017_hessian_fault : Test(1) {    #{{{
+sub t040_hessian_fault : Test(1) {    #{{{
     my $self         = shift;
     my $deserializer = $self->{deserializer};
-    my $hessian_data = "FH\x04code\x10ServiceException\x07message"
+    my $hessian_data = "f\x04code\x10ServiceException\x07message"
       . "\x0eFile Not Found\x06detailM\x1djava.io.FileNotFoundExceptionZZ";
 
     eval {
@@ -99,21 +114,24 @@ sub t017_hessian_fault : Test(1) {    #{{{
 
 }    #}}}
 
-sub t019_hessian_call : Test(3) {    #{{{
-    my $self         = shift;
-    my $hessian_data = "H\x02\x00C\x02eq\x92M\x07qa.Bean\x03foo\x9dZQ\x90";
-    my $hessian_obj  = Hessian::Translator->new( version => 2 );
+sub t050_hessian_call : Test(3) {    #{{{
+    my $self = shift;
+    my $hessian_data =
+        "c\x02\x00m\x00\x02eq"
+      . "Mt\x00\x07qa.BeanS\x00\x03foo"
+      . "I\x00\x00\x00\x0dzR\x00\x00\x00\x00z";
+    my $hessian_obj = Hessian::Translator->new( version => 2 );
     $hessian_obj->input_string($hessian_data);
 
     my $datastructure = $hessian_obj->process_message();
     cmp_deeply(
-        $datastructure->[1]->{call},
+        $datastructure->{call},
         {
             arguments => ignore(),
             method    => 'eq'
         }
     );
-    my @arguments = @{ $datastructure->[1]->{call}->{arguments} };
+    my @arguments = @{ $datastructure->{call}->{arguments} };
 
     foreach my $argument (@arguments) {
         isa_ok( $argument, 'qa.Bean', "Type parsed from call" );
